@@ -3,14 +3,6 @@
 
 G_BEGIN_DECLS
 
-typedef enum {
-   GW_DICTINSTWINDOW_DICTSTOREFIELD_SHORT_NAME,
-   GW_DICTINSTWINDOW_DICTSTOREFIELD_LONG_NAME,
-   GW_DICTINSTWINDOW_DICTSTOREFIELD_DICTINST_PTR,
-   GW_DICTINSTWINDOW_DICTSTOREFIELD_CHECKBOX_STATE, 
-   TOTAL_GW_DICTINSTWINDOW_DICTSTOREFIELDS
-} GwDictionaryStoreField;
-
 typedef enum {
   GW_DICTINSTWINDOW_ENGINESTOREFIELD_ID,
   GW_DICTINSTWINDOW_ENGINESTOREFIELD_NAME,
@@ -34,8 +26,7 @@ struct _GwDictionaryInstallWindowPrivate {
   GtkListStore *encoding_store;
   GtkListStore *compression_store;
   GtkListStore *engine_store;
-  GtkListStore *dictionary_store;
-  LwDictInst *di;
+  LwDictionary *dictionary;
 
   GtkTreeView *view;
   GtkButton *add_button;
@@ -50,9 +41,7 @@ struct _GwDictionaryInstallWindowPrivate {
   GtkButton* source_choose_button;
   GtkButton* source_reset_button;
   GtkComboBox* encoding_combobox;
-  GtkComboBox* compression_combobox;
-  GtkCheckButton* split_checkbutton;
-  GtkCheckButton* merge_checkbutton;
+  GtkCheckButton* postprocess_checkbutton;
 };
 
 #define GW_DICTIONARYINSTALLWINDOW_GET_PRIVATE(object)(G_TYPE_INSTANCE_GET_PRIVATE ((object), GW_TYPE_DICTIONARYINSTALLWINDOW, GwDictionaryInstallWindowPrivate))
