@@ -25,8 +25,9 @@
 //! @brief To be written
 //!
 
-
-#include "../private.h"
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
 
 #include <string.h>
 #include <stdio.h>
@@ -37,6 +38,7 @@
 #include <hunspell/hunspell.h>
 
 #include <gwaei/gwaei.h>
+#include <gwaei/gettext.h>
 #include <gwaei/spellcheck-private.h>
 
 static void gw_spellcheck_attach_signals (GwSpellcheck*);
@@ -79,58 +81,29 @@ gw_spellcheck_new_with_entry (GwApplication *application, GtkEntry *entry)
 static gchar**
 gw_spellcheck_get_dictionary_paths ()
 {
-    //Decalrations
-    gchar **relative_path;
+    //Declarations
     gchar **path;
     gchar *text;
-    gchar *temp;
     const gchar * const * system_path = g_get_system_data_dirs ();
     gint i;
 
-    //Initializations
-    path = NULL;
-
-    relative_path = g_new (gchar*, 4);
-    if (relative_path != NULL)
+    for (i = 0; system_path[i] != NULL; i++)
     {
-      relative_path[0] = g_build_filename ("..", "share", "myspell", "dicts", NULL);
-      relative_path[1] = g_build_filename ("..", "share", "hunspell", NULL);
-      relative_path[2] = g_build_filename ("..", "..", "share", "myspell", "dicts", NULL);
-      relative_path[3] = g_build_filename ("..", "..", "share", "hunspell", NULL);
+      gchar *temp_path1 = g_build_filename (system_path[i], "myspell", NULL);
+      gchar *temp_path2 = g_build_filename (system_path[i], "myspell", "dicts", NULL);
+      gchar *temp_path3 = g_build_filename (system_path[i], "hunspell", NULL);
 
-      text = g_strjoin (":", HUNSPELL_MYSPELL_DICTIONARY_PATH,
-                             relative_path[0],
-                             relative_path[1],
-                             relative_path[2],
-                             relative_path[3],
-                             NULL
-      );
+      text = g_strjoin (":", HUNSPELL_MYSPELL_DICTIONARY_PATH, temp_path1, temp_path2, temp_path3, NULL);
 
-      for (i = 0; system_path[i] != NULL; i++)
-      {
-        gchar *temp_path1 = g_build_filename (system_path[i], "myspell", "dicts", NULL);
-        gchar *temp_path2 = g_build_filename (system_path[i], "hunspell", NULL);
-
-        temp = g_strjoin (":", text, temp_path1, temp_path2, NULL);
-        g_free (text);
-        text = temp;
-        temp = NULL;
-
-        if (temp_path1 != NULL) g_free (temp_path1);
-        if (temp_path2 != NULL) g_free (temp_path2);
-        if (temp != NULL) g_free (temp);
-      }
-      
-      path = g_strsplit (text, ":", -1);
-
-      if (text != NULL) g_free (text); text = NULL;
-      if (relative_path[0] != NULL) g_free (relative_path[0]);
-      if (relative_path[1] != NULL) g_free (relative_path[1]);
-      if (relative_path[2] != NULL) g_free (relative_path[2]);
-      if (relative_path[3] != NULL) g_free (relative_path[3]);
-      g_free (relative_path); relative_path = NULL;
+      g_free (temp_path1);
+      g_free (temp_path2);
+      g_free (temp_path3);
     }
 
+    path = g_strsplit (text, ":", -1);
+
+    g_free (text);
+
     return path;
 }
 
@@ -473,10 +446,10 @@ gw_spellcheck_set_entry (GwSpellcheck *spellcheck, GtkEntry *entry)
         g_signal_handler_disconnect (G_OBJECT (priv->entry), priv->signalid[GW_SPELLCHECK_SIGNALID_DRAW]);
 
       if (priv->signalid[GW_SPELLCHECK_SIGNALID_CHANGED] != 0)
-        g_signal_handler_disconnect (G_OBJECT (priv->entry), priv->signalid[GW_SPELLCHECK_SIGNALID_DRAW]);
+        g_signal_handler_disconnect (G_OBJECT (priv->entry), priv->signalid[GW_SPELLCHECK_SIGNALID_CHANGED]);
 
       if (priv->signalid[GW_SPELLCHECK_SIGNALID_POPULATE_POPUP] != 0)
-        g_signal_handler_disconnect (G_OBJECT (priv->entry), priv->signalid[GW_SPELLCHECK_SIGNALID_DRAW]);
+        g_signal_handler_disconnect (G_OBJECT (priv->entry), priv->signalid[GW_SPELLCHECK_SIGNALID_POPULATE_POPUP]);
 
       if (priv->signalid[GW_SPELLCHECK_SIGNALID_DESTROY] != 0)
         g_signal_handler_disconnect (G_OBJECT (priv->entry), priv->signalid[GW_SPELLCHECK_SIGNALID_DESTROY]);
