@@ -23,8 +23,9 @@
 //! @file libwaei.c
 //!
 
-
-#include "../private.h"
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
 
 #include <stdlib.h>
 #include <string.h>
@@ -35,3 +36,4 @@
 #include <glib-object.h>
 
 #include <libwaei/libwaei.h>
+#include <libwaei/gettext.h>
